import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import omit from 'lodash/omit';
import chunk from 'lodash/chunk';
import classNames from 'classnames';
import Collapse from 'react-collapse';

import Item from './Item';
import './collapsible.scss';

const propTypes = {
  className: PropTypes.string,
  children: PropTypes.func.isRequired,
  variant: PropTypes.oneOf(['cards', 'accordion', 'accordion-block']),
  items: PropTypes.arrayOf(PropTypes.object).isRequired,
  itemRender: PropTypes.func.isRequired,
  columns: PropTypes.oneOf(['12', '11', '10', '9', '8', '7', '6', '5', '4', '3', '2']),
  selectedId: PropTypes.string,
  refId: PropTypes.string,
  component: PropTypes.oneOfType([PropTypes.string, PropTypes.object, PropTypes.func]),
  duration: PropTypes.number,
  withHashTag: PropTypes.bool,
  contentWithoutPadding: PropTypes.bool,
  scrollJump: PropTypes.bool,
  collapse: PropTypes.bool
};

const defaultProps = {
  className: undefined,
  duration: 500,
  component: 'div',
  variant: 'cards',
  columns: '4',
  selectedId: undefined,
  refId: 'id',
  children: null,
  withHashTag: false,
  contentWithoutPadding: false,
  scrollJump: false,
  collapse: true
};

class Collapsible extends PureComponent {
  state = {
    selectedId: undefined
  };

  handlerClick = item => () => {
    const { refId } = this.props;
    const { selectedId: oldSelectedId } = this.state;
    const id = item[refId];
    const selectedId = typeof this.state.selectedId !== 'undefined' ? this.state.selectedId : this.props.selectedId;
    this.setState({
      selectedId: selectedId !== id ? id : false,
      oldSelectedId
    });
  };

  cardsRender() {
    const { items: initialItems, columns } = this.props;
    const itemJunks = chunk(initialItems, parseInt(columns, 10));
    return itemJunks.reduce((resultArray, items, contentRow) => {
      items.forEach((item, row) => {
        resultArray.push(this.renderTrigger({ item, row }));
      });
      resultArray.push(this.renderPlainContent({ items, contentRow }));
      return resultArray;
    }, []);
  }

  accordionRender() {
    const { items } = this.props;
    return items.map((item, row) => (
      <div styleName="accordion-wrapper">
        {this.renderTrigger({ item, row })}
        {this.renderAccordionPlainContent({ item })}
      </div>
    ));
  }

  renderTrigger({ item, row }) {
    const { itemRender, columns: initialColumns, scrollJump, duration, refId, variant } = this.props;
    const selectedId = typeof this.state.selectedId !== 'undefined' ? this.state.selectedId : this.props.selectedId;
    const columns = parseInt(initialColumns, 10);
    const resultItem = {
      ...item,
      active: selectedId === item[refId],
      handlerClick: this.handlerClick
    };
    const isSelected = selectedId === item[refId];
    let styleName = '';
    let buttonVariant = 'button-collapsible-card';
    switch (variant) {
      case 'cards':
        styleName = classNames(`card-${columns}`, { 'last-card': row + 1 === columns });
        break;
      case 'accordion':
        styleName = classNames('trigger-accordion', { 'is-selected-accordion': isSelected });
        buttonVariant = 'icon';
        break;
      case 'accordion-block':
        styleName = classNames('trigger-accordion-block', { 'is-selected-accordion': isSelected });
        buttonVariant = 'icon';
        break;
      default:
    }

    return (
      <Item
        key={item[refId]}
        to={`content-row-${item[refId]}`}
        id={`content-row-${item[refId]}`}
        onClick={this.handlerClick(item)}
        styleName={styleName}
        variant={buttonVariant}
        active={isSelected}
        scrollJump={scrollJump}
        duration={duration}
      >
        {itemRender(resultItem)}
      </Item>
    );
  }

  renderAccordionPlainContent({ item }) {
    const { children, collapse, refId, contentWithoutPadding } = this.props;
    const { oldSelectedId } = this.state;
    const selectedId = typeof this.state.selectedId !== 'undefined' ? this.state.selectedId : this.props.selectedId;
    const isSelected = item[refId] === selectedId;
    const isOldSelected = item[refId] === oldSelectedId;

    const ComponentWrapper = collapse ? Collapse : 'div';
    return (
      (collapse || !!isSelected) && (
        <ComponentWrapper isOpened={!!isSelected}>
          <div styleName={classNames('plain-content', { 'plain-content-without-padding': contentWithoutPadding })}>
            {(isSelected || isOldSelected) && children(item)}
          </div>
        </ComponentWrapper>
      )
    );
  }

  renderPlainContent({ items, contentRow }) {
    const { children, collapse, refId } = this.props;
    const { oldSelectedId } = this.state;
    const selectedId = typeof this.state.selectedId !== 'undefined' ? this.state.selectedId : this.props.selectedId;
    const selectedItem = items.find(item => item[refId] === selectedId);
    const oldSelectedItem = items.find(item => item[refId] === oldSelectedId);

    const ComponentWrapper = collapse ? Collapse : 'div';
    return (
      <div styleName="plain-content-wrapper" key={`content-row-${contentRow}`}>
        {(collapse || !!selectedItem) && (
          <ComponentWrapper isOpened={!!selectedItem}>
            <div styleName="plain-content">
              {(selectedItem || oldSelectedItem) && children(selectedItem || oldSelectedItem)}
            </div>
          </ComponentWrapper>
        )}
      </div>
    );
  }

  render() {
    const { component: Component, className, variant, ...rest } = this.props;
    return (
      <Component
        className={className}
        styleName={classNames('base', variant)}
        onClick={this.handleClick}
        {...omit(rest, Object.keys(propTypes))}
      >
        {variant === 'cards' && this.cardsRender()}
        {(variant === 'accordion' || variant === 'accordion-block') && this.accordionRender()}
      </Component>
    );
  }
}

Collapsible.propTypes = propTypes;
Collapsible.defaultProps = defaultProps;

export default Collapsible;
