import React from 'react';
import PropTypes from 'prop-types';
import omit from 'lodash/omit';
import classNames from 'classnames';
import Hint from './Hint/index';
import Icon from '../Icon/index';
import Tooltip from '../Tooltip/index';

import './field-control.scss';

const formatErrorLabel = ({ errorValue, label, name }) => (
  errorValue.replace(`"${name}"`, `"${label}"`)
);

const propTypes = {
  className: PropTypes.string,
  label: PropTypes.string,
  children: PropTypes.oneOfType([
    PropTypes.node,
    PropTypes.func
  ]),
  mandatory: PropTypes.bool,
  length: PropTypes.number,
  maxLength: PropTypes.number,
  touched: PropTypes.bool,
  error: PropTypes.node,
  hint: PropTypes.node,
  help: PropTypes.shape({
    overlay: PropTypes.string.isRequired
  }),
  meta: PropTypes.object,
  input: PropTypes.object,
  withoutMarge: PropTypes.bool,
  formatError: PropTypes.func,
  variant: PropTypes.string,
  getError: PropTypes.func
};

const defaultProps = {
  className: null,
  label: null,
  hint: null,
  help: null,
  children: null,
  mandatory: false,
  length: 0,
  maxLength: null,
  touched: false,
  error: null,
  meta: null,
  input: null,
  withoutMarge: false,
  formatError: formatErrorLabel,
  variant: 'primary',
  getError: null
};

const FieldControl = ({
  className,
  label,
  hint,
  help,
  children,
  mandatory,
  maxLength,
  length,
  meta,
  withoutMarge,
  formatError,
  variant,
  getError,
  ...props
}) => {
  const { error, touched } = meta || props;
  const { name } = props.input || props;
  const content = typeof children === 'function' ? children(omit(props, ['error', 'touched'])) : children;
  const checkableVariant = variant === 'checkable';
  const errorValue = error || (getError ? getError(props) : null);
  const errorIsText = typeof errorValue === 'string';
  const contentNode = checkableVariant ? (
    <div styleName={classNames(variant, { 'checkable-ok': !errorValue && checkableVariant })}>
      <div styleName={classNames('content')}>{ content }</div>
    </div>)
    : content;
  return (
    <div styleName={classNames('form-group', { 'with-marge': !withoutMarge })} className={className}>
      {label &&
        <div styleName="form-info">
          <span styleName="form-label">
            {label}
            {mandatory && <span styleName="mandatory">*</span>}
          </span>
          {help &&
            <Tooltip {...help}>
              <Icon size="xs" name="help-circled" />
            </Tooltip>}
        </div>
      }
      {contentNode}
      {touched && !!errorValue && errorIsText
        && <Hint variant="danger">{formatError({ ...props, name, label, errorValue })}</Hint>}
      {
        !!maxLength
        && <Hint>
          Content length :{' '}
          <Hint component="span" variant={length <= maxLength ? 'success' : 'danger'}>
            {length}
          </Hint> / {maxLength} (Max. length)
        </Hint>
      }
      {!!hint && <Hint>{hint}</Hint>}
    </div>
  );
};

FieldControl.propTypes = propTypes;
FieldControl.defaultProps = defaultProps;

export default FieldControl;
