import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Dropzone from 'react-dropzone';
import uniqid from 'uniqid';
import warning from 'warning';
import omit from 'lodash/omit';
import slugify from 'slugify';
import FieldControl from '../FieldControl';
import Alert from '../Alert';
import ListFiles from './ListFiles';
import { getInputProps } from '../_helpers/tools';

import './file-input.scss';

const propTypes = {
  className: PropTypes.string,
  formName: PropTypes.string,
  children: PropTypes.func,
  fileRequest: PropTypes.func.isRequired,
  originalFileRequest: PropTypes.func.isRequired,
  onProgress: PropTypes.func,
  onDrop: PropTypes.func,
  onDelete: PropTypes.func,
  proxyGetValue: PropTypes.func,
  proxyChangeValue: PropTypes.func,
  accept: PropTypes.string,
  maxSize: PropTypes.number,
  minSize: PropTypes.number,
  maxFiles: PropTypes.number,
  input: PropTypes.object,
  label: PropTypes.string,
  mandatory: PropTypes.bool,
  multiple: PropTypes.bool,
  placeHolder: PropTypes.string,
  autoUpload: PropTypes.bool,
  progressBar: PropTypes.bool,
  allowDelete: PropTypes.bool,
  allowEdit: PropTypes.bool
};

const defaultProps = {
  className: null,
  formName: 'default',
  children: undefined,
  fileRequest: () => {},
  originalFileRequest: () => {},
  onDrop: undefined,
  onDelete: undefined,
  proxyGetValue: undefined,
  proxyChangeValue: undefined,
  onProgress: undefined,
  accept: undefined,
  maxSize: Infinity,
  minSize: 0,
  maxFiles: null,
  input: null,
  label: null,
  mandatory: false,
  multiple: false,
  placeHolder: null,
  autoUpload: false,
  progressBar: false,
  allowEdit: true,
  allowDelete: true
};

const getFileInputProps = initialProps => {
  const { proxyGetValue } = initialProps;
  const props = getInputProps(initialProps);
  if (typeof proxyGetValue === 'function') {
    props.value = proxyGetValue(initialProps);
  } else if (props.value && !Array.isArray(props.value)) {
    props.value = [props.value];
  }
  return props;
};

const changeValue = ({ value, props }) => {
  const { multiple, maxFiles, proxyChangeValue } = props;
  if (typeof proxyChangeValue === 'function') {
    return proxyChangeValue({ value, props });
  }
  const { onChange } = getFileInputProps(props);
  if (onChange) {
    if (!multiple) return onChange(value[0] || null);
    onChange(maxFiles ? value.slice(0, maxFiles) : value);
  }
};

class FileInput extends Component {
  getInputProps = customProps => getFileInputProps(customProps || this.props);
  changeValue = value => changeValue({ value, props: this.props });

  handleDelete = (file, row) => {
    const { onDelete } = this.props;
    const {
      value: [...value]
    } = this.getInputProps();
    value.splice(row, 1);
    if (onDelete) onDelete(value);
    this.changeValue(value);
  };

  handleEdit = (file, row) => {
    const {
      value: [...value]
    } = this.getInputProps();
    value[row] = file;
    this.changeValue(value);
  };

  handleDrop = files => {
    const { onDrop, multiple, autoUpload, maxFiles, progressBar } = this.props;
    const { value = [] } = this.getInputProps();
    let filteredFiles = files;
    if (maxFiles) {
      if (value.length >= maxFiles) return;
      filteredFiles = files.slice(0, maxFiles - value.length);
    }
    const filesToUpload = filteredFiles.map((file, row) => {
      const { name: initialName, type, preview } = file;
      const splitName = initialName.split('.');
      const fileType = splitName.pop();
      const name = splitName.join('.');
      const hashName = `${uniqid()}-${slugify(name)}.${fileType}`;
      let resultFile = {
        hashName,
        file,
        name,
        type,
        preview,
        progressBar,
        status: 'waiting'
      };
      if (autoUpload) {
        resultFile = this.submitFile(resultFile, row);
      }
      return resultFile;
    });
    if (filesToUpload.length) {
      this.changeValue(multiple ? [...filesToUpload, ...value] : filesToUpload);
    }
    if (onDrop && filesToUpload.length) {
      onDrop(filesToUpload, this.handleSubmitFiles);
    }
  };

  handleSubmitFiles = filesToUpload => {
    const { value = [] } = this.getInputProps();
    const resultValue = value.map((fileObject, row) => {
      const selectedFile = filesToUpload.find(
        ({ hashName, status }) =>
          hashName && hashName === fileObject.hashName && status === 'waiting'
      );
      if (selectedFile) {
        return this.submitFile(fileObject, row);
      }
      return fileObject;
    });
    this.changeValue(resultValue);
  };

  submitFile = (fileToUpload, row) => {
    const {
      fileRequest,
      originalFileRequest = this.props.fileRequest
    } = this.props;
    const { file, originalFile } = fileToUpload;
    const data = new FormData();
    data.append('file', file);
    const request = fileRequest({ ...fileToUpload, data }, progress =>
      this.handleProgress({ progress, file })
    );
    if (!request || typeof request.then !== 'function') {
      warning(false, '"uploadFiles" function need return promise');
      return { ...fileToUpload, status: 'error' };
    }
    const requests = [request];
    if (originalFile) {
      const originalData = new FormData();
      originalData.append('file', originalFile);
      const originalRequest = originalFileRequest({
        ...fileToUpload,
        data: originalData
      });
      if (!originalRequest || typeof originalRequest.then !== 'function') {
        warning(false, '"uploadFiles" function need return promise');
        return { ...fileToUpload, status: 'error' };
      }
      requests.push(originalRequest);
    }

    Promise.all(requests)
      .then(([fileResp, originalFileResp]) =>
        this.submitFileSuccess({ fileResp, originalFileResp, fileToUpload })
      )
      .catch(error => this.submitFileError({ error, fileToUpload, row }));
    return { ...fileToUpload, request, status: 'pending', progress: 0 };
  };

  handleProgress = ({ progress, file }) => {
    const { value = [] } = this.getInputProps();
    const result = value.map(fileToUpload => {
      if (file === fileToUpload.file) {
        return { ...fileToUpload, progress };
      }
      return fileToUpload;
    });
    this.changeValue(result);
  };

  submitFileSuccess = ({
    fileResp = {},
    originalFileResp = {},
    fileToUpload: { hashName }
  }) => {
    const { value = [] } = this.getInputProps();
    const { data: fileRespData } = fileResp;
    const { data: originalFileRespData } = originalFileResp;
    const resultValue = value.map(fileObject => {
      if (!hashName || hashName !== fileObject.hashName) return fileObject;
      const { path, type = fileObject.type } = fileRespData;
      const filteredFilObject = omit(fileObject, [
        'file',
        'originalFile',
        'preview',
        'progress',
        'status',
        'request'
      ]);
      if (originalFileRespData || fileObject.originalPath) {
        const {
          path: originalPath = fileObject.originalPath,
          type: originalType = fileObject.originalType
        } = originalFileRespData || {};
        return {
          ...filteredFilObject,
          path,
          type,
          originalPath,
          originalType
        };
      }
      return {
        ...filteredFilObject,
        path,
        type
      };
    });
    this.changeValue(resultValue);
  };

  submitFileError = ({ error, row }) => {
    const { value = [] } = this.getInputProps();
    const resultValue = value.map((fileObject, fileRow) => {
      if (row !== fileRow) return fileObject;
      return {
        ...omit(fileObject, [
          'file',
          'originalFile',
          'preview',
          'progress',
          'status',
          'request'
        ]),
        status: 'error',
        error
      };
    });
    this.changeValue(resultValue);
  };

  renderWithoutChild() {
    const { value } = this.getInputProps();
    return (
      value &&
      !!value.length && (
        <ListFiles
          key="list-files"
          value={value}
          handleDelete={this.handleDelete}
          handleEdit={this.handleEdit}
          allowDelete={this.props.allowDelete}
          allowEdit={this.props.allowEdit}
        />
      )
    );
  }

  render() {
    const {
      children,
      multiple,
      maxSize,
      minSize,
      maxFiles,
      accept,
      placeHolder,
      ...props
    } = this.props;
    const { value = [] } = this.getInputProps();
    return (
      <FieldControl {...props}>
        {({ ...restProps }) => {
          const { name } = this.getInputProps(restProps);
          return [
            <Dropzone
              name={name}
              onDrop={this.handleDrop}
              styleName="drop-zone"
              multiple={multiple}
              maxSize={maxSize}
              minSize={minSize}
              accept={accept}
              key="drop-zone"
            >
              {({ isDragActive, isDragReject }) => {
                let dropWording;
                switch (true) {
                  case isDragActive && maxFiles && value.length >= maxFiles:
                    dropWording = (
                      <Alert type="warning">The maximum files is exceed</Alert>
                    );
                    break;
                  case isDragReject:
                    dropWording = (
                      <Alert type="warning">This file is not authorized</Alert>
                    );
                    break;
                  case isDragActive:
                    dropWording = (
                      <Alert type="info">This file is authorized</Alert>
                    );
                    break;
                  default:
                    dropWording = (
                      <p styleName="placeholder">
                        {' '}
                        {placeHolder || 'You can drop your file(s) here'}
                      </p>
                    );
                }

                return <div key="files-zone">{dropWording}</div>;
              }}
            </Dropzone>,
            children ? (
              <div key="children">
                {children({
                  ...this.getInputProps(restProps),
                  handleSubmitFiles: this.handleSubmitFiles,
                  handleDelete: this.handleDelete
                })}
              </div>
            ) : (
              this.renderWithoutChild()
            )
          ];
        }}
      </FieldControl>
    );
  }
}

FileInput.propTypes = propTypes;
FileInput.defaultProps = defaultProps;

FileInput.ListFiles = ListFiles;
FileInput.changeValue = changeValue;
FileInput.getFileInputProps = getFileInputProps;

export default FileInput;
