import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Select from 'react-select';

import './filter-bar.scss';

class FilterBar extends Component {
  static propTypes = {
    /**
     * Input value.
     * Exemples: {}, { country: 'fr' }, { roles: [1, 2, 3] }
     */
    value: PropTypes.object.isRequired,
    /**
     * Each options has to contain sub options and optionaly
     * a loadSubOptions() function to update them.
     */
    baseOptions: PropTypes.array.isRequired,
    /**
     * Takes the filters (Array) as sole argument.
     */
    onChange: PropTypes.func.isRequired,
    /**
     * Allow end user to select multiple times the same
     * base option but with a different sub otpion.
     */
    allowMultipleSameProp: PropTypes.bool,
    placeholder: PropTypes.string
  };

  static defaultProps = {
    allowMultipleSameProp: false,
    placeholder: 'Filter...'
  };

  state = {
    incompleteFilter: null
  };

  getOptions = () => {
    const { baseOptions } = this.props;
    const { incompleteFilter } = this.state;
    if (!incompleteFilter) return baseOptions;
    const baseOption = baseOptions.find(o => o.value === incompleteFilter.value);
    return baseOption.subOptions || [];
  };

  handleFilterChange = select => {
    const [lastSelect, beforeLastSelect, ...rest] = [...select].reverse();

    // User hitted delete button/key and either
    // there is no option remaining,
    // or last selected item is a final option (already merged).
    if (!select.length || lastSelect.prop) {
      this.setState({ incompleteFilter: null });
      return this.props.onChange(this.formatOutput(select));
    }

    // Last selected item is a base option.
    if (lastSelect.subOptions) {
      // This base option need to ansynchronously load his sub options.
      if (lastSelect.loadSubOptions) {
        lastSelect.loadSubOptions();
      }
      // Do not call onChange since it's an intermediary state.
      return this.setState({
        incompleteFilter: { ...lastSelect, label: `${lastSelect.label}:` }
      });
    }

    // Last selected item is a sub option and beforeLastSelect is a base option.
    // Merge base option with sub option to form one final otpion.
    const newSelect = [
      ...(this.props.allowMultipleSameProp ? rest : rest.filter(o => o.prop !== beforeLastSelect.value)),
      {
        prop: beforeLastSelect.value,
        value: lastSelect.value,
        label: `${beforeLastSelect.label} ${lastSelect.label}`
      }
    ];
    this.setState({ incompleteFilter: null });
    this.props.onChange(this.formatOutput(newSelect));
  };

  handleFilterInputChange = input => {
    const { incompleteFilter } = this.state;
    if (incompleteFilter && incompleteFilter.loadSubOptions) {
      incompleteFilter.loadSubOptions(input);
    }
    return input;
  };

  findLabel = filter => {
    const { baseOptions } = this.props;
    const baseOption = baseOptions.find(({ value }) => value === filter.prop);
    if (!baseOption) return '';
    const subOption = baseOption.subOptions.find(
      ({ value }) => value === filter.value
    ) || {};
    return `${baseOption.label}: ${subOption.label || filter.value}`;
  };

  formatOutput = filters => filters.reduce((acc, filter) => {
    if (acc[filter.prop]) {
      const currentValue = Array.isArray(acc[filter.prop])
        ? acc[filter.prop]
        : [acc[filter.prop]];
      return {
        ...acc,
        [filter.prop]: [
          ...currentValue,
          filter.value
        ]
      };
    }
    return {
      ...acc,
      [filter.prop]: filter.value
    };
  }, {});

  formatInput = filters =>
    Object.entries(filters).reduce((acc, [prop, values]) => {
      if (Array.isArray(values)) {
        return [
          ...acc,
          ...values.map(value => ({
            prop,
            value,
            label: this.findLabel({ prop, value })
          }))
        ];
      }
      return [
        ...acc,
        {
          prop,
          value: values,
          label: this.findLabel({ prop, value: values })
        }
      ];
    }, []);

  render() {
    const { value, placeholder } = this.props;
    const { incompleteFilter } = this.state;
    const formattedValue = this.formatInput(value);
    return (
      <Select
        styleName="filter"
        multi
        placeholder={placeholder}
        value={incompleteFilter ? [...formattedValue, incompleteFilter] : formattedValue}
        options={this.getOptions()}
        onChange={this.handleFilterChange}
        onInputChange={this.handleFilterInputChange}
      />
    );
  }
}

export default FilterBar;
