import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import Item from './Item/index';
import SubMenu from './SubMenu/index';
import BurgerButton from './BurgerButton/index';
import AppTitle from './AppTitle/index';

import './side-menu.scss';

const propTypes = {
  /** children is a function, has closeMenu func as prop */
  children: PropTypes.func.isRequired,
  /** @ignore */
  className: PropTypes.string,
  /** @ignore */
  side: PropTypes.oneOf(['left', 'right']),
  size: PropTypes.oneOf(['sm', 'md', 'lg']),
  variant: PropTypes.oneOf(['light', 'dark']),
  /** A component: ul, div etc... */
  component: PropTypes.oneOfType([PropTypes.string, PropTypes.object, PropTypes.func]),
  /** sizes where collapsible behavior is expected (ex: "xs sm") */
  collapsible: PropTypes.string,
  /** true => menu is hidden when not collapsible */
  onlyCollapsible: PropTypes.bool,
  hasBurgerButton: PropTypes.bool,
  /** if burgerButton variant different from menu variant */
  burgerButtonVariant: PropTypes.oneOf(['light', 'dark']),
  burgerButtonStyle: PropTypes.object,
  /** controller for the open state of the Side Menu */
  open: PropTypes.bool
};

const defaultProps = {
  className: undefined,
  size: 'md',
  variant: 'light',
  open: false,
  side: 'left',
  collapsible: null,
  hasBurgerButton: true,
  component: 'ul',
  onlyCollapsible: false,
  burgerButtonVariant: undefined,
  burgerButtonStyle: undefined
};

const contextTypes = {
  layout: PropTypes.object
};

class SideMenu extends PureComponent {
  state = {
    open: false,
    buttonHidden: ['xs', 'sm', 'md', 'lg', 'xl'],
    collapsibleSizes: []
  };

  componentWillMount = () => {
    const { side, collapsible, onlyCollapsible, size } = this.props;
    const buttonHidden = this.state.buttonHidden;
    const collapsibleSizes = collapsible
      ? collapsible.split(' ').map(screensize => {
        buttonHidden.splice(buttonHidden.indexOf(screensize), 1);
        return `collapsible-${screensize}`;
      })
      : collapsible;
    this.setState({ ...this.state, buttonHidden, collapsibleSizes });
    if (this.context.layout) {
      this.context.layout.register('sideColumn', { side, size, collapsible: collapsible.split(' '), onlyCollapsible });
    }
  };

  toggleMenu = (e, newState = !this.state.open) => {
    this.setState({ ...this.state, open: newState });
  };

  /**
   * A responsive Side Menu, with SubMenus & Items
   * (plus the BurgerButton for the responsive Navbar and the AppTitle for the "desktop" Navbar)
   */
  render() {
    const {
      children,
      className,
      size,
      variant,
      side,
      onlyCollapsible,
      hasBurgerButton,
      burgerButtonVariant,
      burgerButtonStyle,
      component: Component,
      ...rest
    } = this.props;
    const { buttonHidden, collapsibleSizes } = this.state;
    const { layout } = this.context;
    return (
      <div
        className={className}
        styleName={classNames(side, collapsibleSizes, variant, {
          'state-open': hasBurgerButton ? this.state.open : this.props.open,
          'only-collapsible': onlyCollapsible
        })}
      >
        <div // eslint-disable-line jsx-a11y/no-static-element-interactions
          styleName={classNames('overlay', variant)}
          onClick={this.toggleMenu}
        />

        {hasBurgerButton && (
          <BurgerButton
            {...{ side }}
            variant={burgerButtonVariant || variant}
            hidden={buttonHidden.join(' ')}
            open={this.state.open}
            onClick={this.toggleMenu}
            style={burgerButtonStyle}
          />
        )}
        <div
          styleName={classNames('base', size, side, {
            'with-header': layout && layout.config && layout.config.header && layout.config.header.fixed
          })}
          {...rest}
        >
          <Component styleName={classNames('list')}>{children({ closeMenu: this.toggleMenu })}</Component>
        </div>
      </div>
    );
  }
}

SideMenu.propTypes = propTypes;
SideMenu.defaultProps = defaultProps;
SideMenu.contextTypes = contextTypes;

SideMenu.Item = Item;
SideMenu.SubMenu = SubMenu;
SideMenu.BurgerButton = BurgerButton;
SideMenu.AppTitle = AppTitle;

export default SideMenu;
