import React, { PureComponent, Children, isValidElement } from 'react';
import PropTypes from 'prop-types';
import { Sticky } from 'react-sticky';
import omit from 'lodash/omit';
import classNames from 'classnames';
import Section from '../../Section';

import './tabs-container.scss'; // eslint-disable-line no-unused-vars

const { Column } = Section;

const propTypes = {
  children: PropTypes.node,
  scrollSpy: PropTypes.object,
  sticky: PropTypes.object,
  withHashTag: PropTypes.bool,
  className: PropTypes.string,
  tabDirection: PropTypes.oneOf(['column', 'row']),
  backgroundWhite: PropTypes.bool,
  variant: PropTypes.string
};

const defaultProps = {
  children: undefined,
  scrollSpy: null,
  sticky: null,
  withHashTag: false,
  className: undefined,
  backgroundWhite: false,
  tabDirection: 'column',
  variant: undefined
};

class TabsContainer extends PureComponent {

  state = {
    selectedIndex: -1,
    targetItems: [],
    inViewState: []
  };

  targetChildren = {}; // eslint-disable-line react/sort-comp
  timer = null; // eslint-disable-line react/sort-comp

  componentDidMount() {
    const { scrollSpy } = this.props;
    if (scrollSpy) {
      this._initFromProps(this.props);
      window.addEventListener('scroll', this._handleSpy);
    }
  }

  componentWillReceiveProps(nextProps) {
    this._initFromProps(nextProps);
  }

  componentWillUnmount() {
    window.removeEventListener('scroll', this._handleSpy);
  }

  onTabsChange = selectedIndex => {
    this.setState({
      selectedIndex
    });
  };

  _getElemsViewState = () => {
    const inView = [];
    const outView = [];
    const viewStatusList = [];
    const { targetItems } = this.state;
    let hasInViewAlready = false;

    for (let i = 0, max = targetItems.length; i < max; i += 1) {
      if (!this.targetChildren[targetItems[i]]) {
        this.targetChildren[targetItems[i]] = document.getElementById(targetItems[i]);
      }
      const currentContent = this.targetChildren[targetItems[i]];
      const isInView = hasInViewAlready ? false : this._isInView(currentContent, i);
      if (isInView) {
        hasInViewAlready = true;
        inView.push(currentContent);
      } else {
        outView.push(currentContent);
      }
      viewStatusList.push(isInView);
    }

    return { inView, outView, viewStatusList };
  };

  _isInView = el => {
    if (!el) return false;
    const { topOffset = 0 } = this.props.scrollSpy || {};
    const { top, height } = el.getBoundingClientRect();
    return ((top + height + topOffset) > 0 || top > 0);
  };

  _handleSpy = () => {
    if (this.timer) {
      clearTimeout(this.timer);
    }
    this.timer = setTimeout(() => this.setState({ inViewState: this._getElemsViewState().viewStatusList }), 20);
  };

  _initFromProps = props => {
    const { children } = props || this.props;
    const targetItems = Children
      .toArray(children)
      .reduce((resultArray, child) => {
        if (isValidElement(child)) {
          const { props: { to = '', href = '' }, type: { componentName } } = child;
          if (componentName === 'Tab') {
            const target = to.replace('#', '') || href.replace('#', '');
            if (resultArray.indexOf(target) === -1) resultArray.push(target);
          }
        }
        return resultArray;
      }, []) || [];
    this.targetChildren = {};
    this.setState({
      targetItems
    });
    this._handleSpy();
  };

  renderNav({ style } = {}) {
    const { children, scrollSpy, withHashTag, tabDirection, variant, ...props } = this.props;
    const { topOffset = 0 } = scrollSpy || {};
    const { selectedIndex, inViewState } = this.state;
    delete props.topOffset;
    let i = 0;
    return (
      <nav
        style={{ ...style, zIndex: 10 }}
        styleName={classNames(tabDirection, variant)}
      >
        {Children.map(children, child => {
          if (!isValidElement(child)) {
            return child;
          }
          const { type: { componentName } } = child;
          let myChild;
          if (componentName === 'Tab') {
            const childProps = {
              tabDirection,
              index: i
            };
            if (variant) {
              childProps.variant = variant;
            }
            if (scrollSpy) {
              myChild = React.cloneElement(child, {
                ...childProps,
                active: inViewState[i],
                offset: topOffset,
                scrollSpy,
                withHashTag,
                duration: 300
              });
            } else {
              childProps.onChange = this.onTabsChange;
              if (selectedIndex !== -1) {
                childProps.active = i === selectedIndex;
              }
              myChild = React.cloneElement(child, childProps);
            }
            i += 1;
            return myChild;
          }
          return child;
        })}
      </nav>
    );
  }

  render() {
    const { sticky, tabDirection, className, backgroundWhite, ...props } = this.props;
    let TabsContainerComponent;
    switch (tabDirection) {
      case 'column':
        TabsContainerComponent = Column;
        break;
      case 'row':
      default:
        TabsContainerComponent = 'div';
    }

    return (
      <TabsContainerComponent
        className={className}
        styleName={classNames({ 'background-white': backgroundWhite })}
        {...omit(props, Object.keys(propTypes))}
      >
        {sticky ?
          <Sticky {...sticky}>
            {({ distanceFromBottom, style: { top = 0, ...style } }) => {
              const topOffset = -1 * sticky.topOffset;
              const styleTopOffset = Math.max(
                0, topOffset > distanceFromBottom ? topOffset - (topOffset - distanceFromBottom) : topOffset
              );
              style.top = top + styleTopOffset; // eslint-disable-line no-param-reassign
              return this.renderNav({ style });
            }}
          </Sticky>
          : this.renderNav()}
      </TabsContainerComponent>
    );
  }
}

TabsContainer.propTypes = propTypes;
TabsContainer.defaultProps = defaultProps;
TabsContainer.componentName = 'TabsContainer';

export default TabsContainer;
